TrailingNewlinePolicy

Enum TrailingNewlinePolicy 

pub enum TrailingNewlinePolicy {
    Never,
    IfMultiline,
    Always,
}

Variants



Never

Never produces a newline at the end of EDN



IfMultiline

Produces a newline at the end of EDN if there are any internal newlines



Always

Always produces a newline at the end of EDN